///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2012 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//      TF  Jan 03, 2012    Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import java.io.FileInputStream;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbFieldType;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbTextExtractionInfo;
import com.tietoenator.trip.jxp.data.TdbTextField;
import com.tietoenator.trip.jxp.session.TdbSession;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;


/**
 * This class demonstrates how to use TRIPcof text extraction on the server
 * and locally. For server-side use, TRIPcof must be installed together with
 * TRIPsystem. For client-side use, TRIPcof must be installed on the machine
 * on which this example program is being run.
 *
 * <p>Use the TRIPcof sample database with this example program. Database
 * definition files for it are available in the TRIPcof installation.</p>
 *
 * <p><b>NOTE:</b>
 * To run this program with client-side processing, you must first define the
 * Java system property or environment variable TRIPCOF_HOME to be the
 * fully qualified path to the installation directory of TRIPcof on the local
 * machine.
 * </p>
 */
public class CofExtract
{
    private TdbSession m_session;
    private String m_database = null;
    private String m_stringField = null;
    private String m_textField = null;
    private String m_pnameField = null;
    private String m_pvalueField = null;

    /**
     * Constructor
     * @throws TdbException
     */
    public CofExtract()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIPcof Text Extraction Example ****", false);
    }

    void close()
    {
        if (m_session != null)
        {
            try { m_session.logout(); } catch (Exception ignore) {}
            m_session = null;
        }
    }

    private void setDatabase(String db)
    {
        m_database = db;
    }

    private void setStringField(String sf)
    {
        m_stringField = sf;
    }

    private void setTextField(String tf)
    {
        m_textField = tf;
    }

    private void setPNameField(String pn)
    {
        m_pnameField = pn;
    }

    private void setPValueField(String pn)
    {
        m_pvalueField = pn;
    }

    /**
     * Perform text extraction or HTML conversion as specified.
     */
    public void process()
            throws Exception
    {
        TdbRecord rec = null;

        Console stdin = new Console();
        String name = null;
        while (name == null)
            name = stdin.readln("Name of file to extract text from", null, true);

        String namefield = null;
        while (namefield == null)
            namefield = stdin.readln("Field for file name", "FILE_NAME", true);


        rec = new TdbRecord(m_session, m_database, false);

        TdbTextField fld = (TdbTextField) rec.getHead().createField(m_textField, TdbFieldType.TextField);
        TdbTextExtractionInfo txi = fld.getTextExtractionInfo();

        // The optional string field into which the file is to be stored.
        if (m_stringField != null)
        {
            txi.setBinaryCopyField(m_stringField);
        }

        // The optional phrase field that receieves property name/value pairs.
        if (m_pnameField != null && m_pvalueField != null)
        {
            txi.setPropertyNameField(m_pnameField);
            txi.setPropertyValueField(m_pvalueField);
        }

        // Enable client-side extraction?
        if (0 != stdin.readint("Client-side Extraction (0=no, 1=yes)", 0, true))
            txi.setClientSide(true);

        // In contrast to TRIPview and TRIPview-C, the name is an extremely important 
        // peice of information. You must always supply the file name, even if the file
        // to be processed is only present in memory! Also, you are recommended
        // to store the name of the file into the record as well for future
        // reference (e.g. for use by TRIPcof HTML conversion).
        txi.setFileName(name);
        rec.getHead().createField(namefield, TdbFieldType.PhraseField).appendValue(name);

        // Read file data using a stream? Note that if client-side operation is
        // performed, the FileName property (as set above) is sufficient.
        // Passing file data via a stream to JXP should only be done if
        //   1) the file is in memory only, or
        //   2) if TRIPcof and the application are running on two different
        //      machines.
        if (txi.getClientSide() == false)
            txi.setStream(new FileInputStream(name));

        // Remember to explicitly state that text extraction is to be performed.
        txi.setExtractText(true);

        // Send the request to the server.
        rec.commit();

        System.out.println();
        System.out.println("Document successfully stored in record #"+
                Integer.toString(rec.getRecordId()));
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        CofExtract program = null;
        try
        {
            program = new CofExtract();
            String db = null, txt = null, pname = null, pval = null;
            Console stdin = new Console();

            // Get a valid database name, and set the list of
            // retrieval fields from it
            while (db == null)
                db = stdin.readln("TRIPcof Sample Database", null);

            while (txt == null)
                txt = stdin.readln("Text field", "FILE_TEXT", true);

            program.setDatabase(db);
            program.setTextField(txt);

            program.setStringField(stdin.readln("String field (optional)", null));

            pname = stdin.readln("Phrase field for property names (opt)", null);
            if (pname != null)
            {
                program.setPNameField(pname);
                while (pval == null)
                {
                    pval = stdin.readln("Phrase field for property values", null);
                }
                program.setPValueField(pval);
            }

            // Perform the operation
            program.process();
        }
        catch (TdbException e)
        {
            System.out.println();
            System.out.println("[ERROR]");
            System.out.println(e.getDebugMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (program != null)
            {
                program.close();
                program = null;
            }
        }
    }
}
