///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2011 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Jan 24, 2011	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import org.w3c.dom.Document;

import java.io.OutputStream;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;


import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.data.TdbDataFormat;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.data.TdbRecordSet;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.session.TdbSession;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import java.io.FileOutputStream;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * This class demonstrates how to use the XPATH functionality introduced with
 * TRIPxml 3.0 in order to query a TRIPxml database and fetch results as a
 * fragment set.
 *
 * This example requires minimum TRIPsystem 6.2-5 and TRIPxml 3.0 in order to
 * work.
 */
public class XPath
{

    private TdbSession m_session;
    private TdbDatabaseDesign m_design;

    public XPath()
            throws TdbException
    {
        m_session = SessionFactory.newInstance("**** TRIPxml XPath Example ****", false);
    }

    public void close()
    {
        m_design = null;
        try
        {
            if (null != m_session)
            {
                m_session.logout();
            }
        }
        catch (Exception ignore)
        {
        }
        m_session = null;
    }

    TdbDatabaseDesign selectDatabase(Console stdin) throws TdbException
    {
        String db;
        TdbDatabaseDesign design = null;

        do
        {
            db = null;
            while (db == null)
            {
                db = stdin.readln("XML Database", null);
            }

            design = new TdbDatabaseDesign(m_session);
            design.get(db);

            // NOTE: Versions older than 6.2-0 does not return the xml indicator
            // unless we are logged on as the database owner.

            if (!design.isXmlEnabled())
            {
                System.err.println("\nERROR: The database " + design.getName().toUpperCase() + " is not XML enabled.\n");
                design = null;
            }
        }
        while (design == null);
        return design;
    }

    void process() throws Exception
    {
        Console stdin = new Console();
        String strQuery = null;
        String strFragmentExpr = null;
        String strFile = null;
        boolean bHighlight = false;
        boolean bXPathQuery = true;
        int nTo, nFrom;

        // Select an XML database.
        m_design = selectDatabase(stdin);

        // Speciy how to look for a record
        if (0 != stdin.readint("Search using CCL (1) or XPath (0)", 0, true))
        {
            bXPathQuery = false;
            while (strQuery == null)
            {
                strQuery = stdin.readln("FIND order (include FIND or FUZZ)", null);
            }
        }
        else
        {
            while (strQuery == null)
            {
                strQuery = stdin.readln("XPath statement for query", null);
            }
        }

        TdbSearch srch = new TdbSearch(m_session);

        // Open the database and clear its search set.
        srch.execute("BASE " + m_design.getName());
        srch.clear();

        // Execute the query statement
        if (bXPathQuery)
        {
            srch.executeXPath(strQuery);
        }
        else
        {
            srch.execute(strQuery);
        }

        // Get the TdbSearchSet object for this result.
        TdbSearchSet ss = srch.getLastSearchSet();

        // Display a "classic" search info line
        System.out.println();
        System.out.println("S=" + Integer.toString(ss.getSearchId()) + "\t" +
                "<" + Integer.toString(ss.getRecordCount()) + ">\t" +
                ss.getParsedCommand());
        System.out.println();

        // Retrieve result as a fragment set?
        if (0 != stdin.readint("Get result as fragment set? ( Yes=1 No=0 )", 0, true))
        {
            while (strFragmentExpr == null)
            {
                strFragmentExpr = stdin.readln("XPath statement for fragments", null);
            }

            // Should hit terms in TEXT nodes decorated with hit markup?
            if (0 != stdin.readint("Apply hit markup? ( Yes=1 No=0 )", 0, true))
                bHighlight = true;

            // Range of records to get fragments from.
            nFrom = stdin.readint("First record in search to fetch?", 1, true);
            nTo = stdin.readint("Last record in search to fetch?", ss.getRecordCount(), true);

            // Note that a record may yield any number of fragments, all
            // depending on the node set that the XPath expression evaulates
            // to. If the node set is empty for a record, no fragments will be
            // returned from that record.

            Document fragmentDoc = ss.asFragmentSet(strFragmentExpr, nFrom, nTo, bHighlight);

            // Check how many fragments we got.
            String ns = "http://www.tieto.com/trip/xml";
            NodeList fragmentNodes = fragmentDoc.getElementsByTagNameNS(ns,"FRAGMENT");
            System.out.println("The fragment set contains "+
                    Integer.toString(fragmentNodes.getLength())+" fragments.");

            if (0 != stdin.readint("Write result to file? ( Yes=1 No=0 )", 0, true))
            {
                while (strFile == null)
                {
                    strFile = stdin.readln("File name", null);
                }
                serialize(fragmentDoc,new FileOutputStream(strFile));
            }
            else
            {
                serialize(fragmentDoc, System.out);
            }
        }

        ss = null;
        srch.close();
    }

    public void serialize(Document doc, OutputStream out) throws TransformerException
    {
        TransformerFactory trfact = TransformerFactory.newInstance();
        Transformer serializer = trfact.newTransformer();
        serializer.setOutputProperty(OutputKeys.INDENT, "yes");
        serializer.transform(new DOMSource(doc), new StreamResult(out));
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        XPath app = null;
        try
        {
            app = new XPath();
            app.process();
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (null != app)
            {
                app.close();
            }
        }
    }
}
