///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2010 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Jan 05, 2010	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.*;
import com.tietoenator.trip.jxp.data.*;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbLocalSession;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

/**
 * This class demonstrates how to use the TRIPxml functionality to import
 * files and documents into a TRIPxml database.
 */
public class XmlImport
{
    private TdbSession m_session;
    private TdbXmlRecord m_rec;
    private TdbDatabaseDesign m_design;

    /**
     * Constructor
     * @throws TdbException
     */
    public XmlImport()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIPxml Import Example ****", false);
    }


    public void close()
    {
        if (m_rec != null)
        {
            // Calling clear is VERY important if HTTP stream upload has been used!
            // If we forget, we will end up with a bunch of hanging threads.

            m_rec.clear(true);
            m_rec = null;
        }
        m_design = null;

        try
        {
            if (null != m_session)
                m_session.logout();
        }
        catch (Exception ignore) {}
        m_session = null;
    }

   void showMessages(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
         System.out.println(msg);
      prv.resetMessages();
   }

   boolean arrayContains(String str, String[] arr)
   {
       for (String item : arr)
       {
           if (item.equalsIgnoreCase(str))
               return true;
       }
       return false;
   }

    /**
     * Perform the import
     */
    public void process() throws TdbException
    {
        m_rec.commit();

        System.out.println();
        System.out.println("Added record #"+Integer.toString(m_rec.getRecordId()));
        System.out.println();

        m_design.index(false,true);
        showMessages(m_design);
    }

    TdbDatabaseDesign selectDatabase(Console stdin) throws TdbException
    {
        String db;
        TdbDatabaseDesign design = null;

        do
        {
            db = null;
            while (db == null)
                db = stdin.readln("XML Database", null);

            design = new TdbDatabaseDesign(m_session);
            design.get(db);

            // NOTE: Versions older than 6.2-0 does not return the xml indicator
            // unless we are logged on as the database owner.

            if (!design.isXmlEnabled())
            {
                System.err.println("\nERROR: The database "+design.getName().toUpperCase()+" is not XML enabled.\n");
                design = null;
            }
        }
        while (design == null);
        return design;
    }

    String selectImportFile(Console stdin) throws TdbException
    {
        File f = null;
        String fname = null;
        do
        {
            while (fname == null)
                fname = stdin.readln("File to import",null);

            f = new File(fname);
            if (!f.exists() || !f.isFile())
            {
                System.err.println("\nERROR: specified file does not exist or is not a file.\n");
                f = null;
            }
        }
        while (f == null);
        return fname;
    }

    void setParameters() throws TdbException, FileNotFoundException
    {
        Console stdin = new Console();

        // Select an XML database.
        m_design = selectDatabase(stdin);

        // Specify file to import
        String importFile = selectImportFile(stdin);

        // Are we going to update a record?
        int nRid = -1;
        while (nRid<0)
            nRid = stdin.readint("Record to update (0 = insert)", 0, true);

        // Are we going to import from a Java stream?
        InputStream stream = null;
        if (0 != stdin.readint("InputStream import ( Yes=1 No=0 )", 0, true))
            stream = new FileInputStream(importFile);

        // Are we going to store a copy of the original file?
        boolean bStoreCopy = true;
        if (0 == stdin.readint("Store copy ( Yes=1 No=0 )", 1, true))
            bStoreCopy = false;

        // Check the type of the file
        TdbXmlRecord.XmlRecordType rtype = TdbXmlRecord.XmlRecordType.UNKNOWN;
        if (importFile.toLowerCase().endsWith(".xml") || importFile.toLowerCase().endsWith(".xsl"))
            rtype = TdbXmlRecord.XmlRecordType.XML;
        else
        {
            if (0 != stdin.readint("Is the file an XML file? ( Yes=1 No=0 )", 0, true))
               rtype = TdbXmlRecord.XmlRecordType.XML;
        }

        // Determine what I/O mode is valid for the current setup.

        String[] validModes;
        if ((m_session instanceof TdbLocalSession) && rtype == TdbXmlRecord.XmlRecordType.XML)
            validModes = new String[]{"inline","stream","file"};
        else if ((m_session instanceof TdbLocalSession) && rtype != TdbXmlRecord.XmlRecordType.XML)
            validModes = new String[]{"inline","file"};
        else if (!(m_session instanceof TdbLocalSession) && rtype == TdbXmlRecord.XmlRecordType.XML)
            validModes = new String[]{"inline","stream"};
        else
            validModes = new String[]{"inline"};

        // Build the prompt string for asking about what I/O mode to select

        StringBuilder strModePrompt = new StringBuilder("I/O Mode (");
        for (int m=0;m<validModes.length;m++)
        {
            if (m>0)
                strModePrompt.append(",");
            strModePrompt.append(validModes[m]);
        }
        strModePrompt.append(")");

        // Select I/O mode

        String strIOMode = null;
        TdbXmlRecord.IOMode ioMode = TdbXmlRecord.IOMode.Inline;
        do
        {
            while (strIOMode == null)
                strIOMode = stdin.readln(strModePrompt.toString(), "inline");
            if (!arrayContains(strIOMode,validModes))
                strIOMode = null;
        }
        while (strIOMode == null);

        if (strIOMode.equalsIgnoreCase("stream"))
            ioMode = TdbXmlRecord.IOMode.Stream;
        else if (strIOMode.equalsIgnoreCase("file"))
            ioMode = TdbXmlRecord.IOMode.File;
        else
            ioMode = TdbXmlRecord.IOMode.Inline;


        // Should the XML document be validated upon import?
        boolean bValidate = false;
        if (rtype == TdbXmlRecord.XmlRecordType.XML)
        {
            // Are we going to store a copy of the original file?
            if (0 != stdin.readint("Validate document? ( Yes=1 No=0 )", 0, true))
                bValidate = true;
        }

        // Create a TdbXmlRecord object (subclass of TdbRecord)
        m_rec = new TdbXmlRecord(m_session,m_design,false);

        // Select record for update
        if (nRid>0)
        {
            m_rec.setRecordId(nRid);
            m_rec.get();
        }

        // Override various defaults
        // Remember: do this *before* calling prepareImport!
        m_rec.setStoreCopy(bStoreCopy);
        m_rec.setValidate(bValidate);
        m_rec.setInputOutputMode(ioMode);

        // Register the request to import
        m_rec.prepareImport(stream, rtype, importFile);

        System.out.println();
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        XmlImport app = null;
        try
        {
            app = new XmlImport();
            app.setParameters();
            app.process();
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (null != app)
                app.close();
        }
    }
}
