///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2013 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Feb 23, 2013	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.graph;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.data.TdbGraphRecord;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbRecordSet;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.graph.TdbGraph;
import com.tietoenator.trip.jxp.graph.TdbGraphPath;
import com.tietoenator.trip.jxp.graph.TdbGraphPathEdge;
import com.tietoenator.trip.jxp.graph.TdbGraphPathVertex;
import com.tietoenator.trip.jxp.graph.TdbGraphSet;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.text.DecimalFormat;
import java.util.List;

/**
 * Example for how to use transitive graph search operations.
 */
public class Transitive 
{
    private TdbSession m_session;
    
    public Transitive()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** Transitive TRIP graph example ****", false);
    }
    
    public void close()
    {
        if (m_session != null)
        {
            
            try
            {
                m_session.logout();
            }
            catch (TdbException ignore)
            {
            }
            finally
            {
                m_session = null;
            }
        }
    }


   // Show any success message that might be forthcoming from the server

   private static void showMessages(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
      {
         System.out.println(msg);
      }
      prv.resetMessages();
   }

    private String readnoblank(Console stdin, String prefix, String defaultValue)
           throws TdbException
    {
       String inval;
       do
       {
           inval = stdin.readln(prefix,defaultValue,true);
       }
       while (inval.length()==0);
       return inval;
   }
    
    private void showEdges(TdbGraphSet gs, boolean weights, int from, int to)
            throws TdbException
    {
        int startRis = from<1?1:from;
        int stopRis = to<1?gs.getEdgeCount():to;
        
        TdbRecord tmpl = new TdbRecord(m_session);
        tmpl.addToTemplate("NAME");
        tmpl.addToTemplate("WEIGHT");
        tmpl.addToTemplate("S_DB");
        tmpl.addToTemplate("S_RID");
        tmpl.addToTemplate("S_UUID");
        tmpl.addToTemplate("T_DB");
        tmpl.addToTemplate("T_RID");
        tmpl.addToTemplate("T_UUID");
        
        TdbRecordSet rs = new TdbRecordSet(m_session);
        rs.setSearchSet(gs.getSearchId());
        rs.setFrom(startRis);
        rs.setTo(stopRis);
        rs.setRetrievalTemplate(tmpl);
        rs.get();
        
        List<TdbRecord> edges = rs.records();
        DecimalFormat df = new DecimalFormat("#.##");
        
        System.out.println();
        System.out.println(Integer.toString(edges.size()) + " edges found.\n");
        
        for (TdbRecord r : edges)
        {
            // For a set from a graph database all record objects are
            // instances of TdbGraphRecord, but it never hurts to be
            // careful.
            if (!(r instanceof TdbGraphRecord))
            {
                System.out.println("WARNING: Record from graph set was not an instance of TdbGraphRecord.");
                continue;
            }
            
            // Only display data from edge records.
            TdbGraphRecord gr = (TdbGraphRecord)r;
            if (!gr.isEdge())
            {
                System.out.println("WARNING: Record from graph set was not an edge.");
                continue;
            }
            
            System.out.print(gr.getSourceDatabase());
            System.out.print(".");
            System.out.print(Integer.toString(gr.getSourceRecordId()));
            
            System.out.print(" --(");
            System.out.print(gr.getName());
            if (weights)
            {
                System.out.print(":");
                System.out.print(df.format(gr.getWeight()));
            }
            System.out.print(")--> ");
            
            System.out.print(gr.getTargetDatabase());
            System.out.print(".");
            System.out.println(Integer.toString(gr.getTargetRecordId()));
        }
    }
    
   public void process(Console stdin) throws TdbException
   {
       TdbGraph graph;
       TdbGraphSet resultSet;
       String graphDbName;
       String targetDbName;
       String sourceDbName;
       String edgeName;
       String restrictQuery;
       int sourceSet;
       int targetSet=0;
       
       TdbSearch normalSearch = new TdbSearch(m_session);
       normalSearch.execute("DEFINE FIND MIN=0");

       // The graph database to use
       graphDbName = readnoblank(stdin,"Graph database","GRAPHBASE");
       
       // Database for source record to explore relations from
       sourceDbName = readnoblank(stdin,"Database for start record","ALICE");       
       normalSearch.execute("BASE "+sourceDbName);
       
       restrictQuery = readnoblank(stdin,"Condition for source records","");
       if (!restrictQuery.toLowerCase().startsWith("find") && 
            !restrictQuery.toLowerCase().startsWith("fuzz") &&
            !restrictQuery.toLowerCase().startsWith("f "))
       {
            restrictQuery = "FIND " + restrictQuery;
       }
       normalSearch.execute(restrictQuery);
       sourceSet = normalSearch.getLastSearchSet().getSearchId();

       edgeName = readnoblank(stdin,"Edge name (relation)","CHILD");
       String[] edgeNames = new String[1];
       edgeNames[0] = edgeName;
       
       boolean reverse = stdin.readbool("Search in reverse direction",false,true);
       boolean allEdges = stdin.readbool("Get all edges traversed",false,true);
       
       int maxDepth = stdin.readint("Max traversal depth", 10, true);
       
       if (stdin.readbool("Use restriction set for target vertices",false,true))       
       {
            // Database for target vertices
            targetDbName = readnoblank(stdin,"Database for target records","ALICE");     
            if (!targetDbName.equalsIgnoreCase(sourceDbName))
                normalSearch.execute("BASE "+targetDbName);

            restrictQuery = readnoblank(stdin,"Condition for target records","");
            if (!restrictQuery.toLowerCase().startsWith("find") && 
                !restrictQuery.toLowerCase().startsWith("fuzz") &&
                !restrictQuery.toLowerCase().startsWith("f "))
            {
                restrictQuery = "FIND " + restrictQuery;
            }
            normalSearch.execute(restrictQuery);
            targetSet = normalSearch.getLastSearchSet().getSearchId();
       }
       
       graph = new TdbGraph(m_session, graphDbName);
       resultSet = graph.getGraphSet().transitive(sourceSet, targetSet, edgeNames,  
               maxDepth, reverse, allEdges);
       showMessages(graph);

       /* Show the result edges */
       showEdges(resultSet, false, -1, -1);

       /* Show result vertices if allEdges flag is false */
       if (!allEdges)
       {
            TdbSearchSet vertexSet;
            if (reverse)
                vertexSet = resultSet.resolveSources();
            else
                vertexSet = resultSet.resolveTargets();
            showMessages(graph);
       
            System.out.println();
            System.out.println(Integer.toString(vertexSet.getRecordCount())+" records found.\n");
            for (TdbRecord r : vertexSet)
            {
                System.out.println(r.getDatabaseName()+"."+Integer.toString(r.getRecordId()));
            }
       }
       
       normalSearch.close();
       graph.close();
   }
   
   
   public static void main(String[] args)
   {
       Transitive app = null;
       Console stdin = new Console();
       try
       {
           app = new Transitive();
           app.process(stdin);
       }
       catch (TdbException tx)
       {
           System.err.println(tx.getDebugMessage());
       }
       catch (Exception ox)
       {
           ox.printStackTrace(System.err);
       }
       finally
       {
           if (app != null)
               app.close();
       }
   }
}
